/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.test.remoting.tests;

import java.net.InetAddress;
import java.util.HashMap;

import javax.management.MBeanServer;

import junit.framework.TestCase;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.jboss.remoting.Client;
import org.jboss.remoting.InvocationRequest;
import org.jboss.remoting.InvokerLocator;
import org.jboss.remoting.ServerInvocationHandler;
import org.jboss.remoting.ServerInvoker;
import org.jboss.remoting.callback.Callback;
import org.jboss.remoting.callback.HandleCallbackException;
import org.jboss.remoting.callback.InvokerCallbackHandler;
import org.jboss.remoting.transport.Connector;
import org.jboss.remoting.transport.PortUtil;
import org.jboss.test.remoting.callback.configuration.CallbackServerConfigurationTestCase;

/**
 * Constructed especially for ALR.
 *  
 * @author <a href="mailto:ron.sigal@jboss.com">Ron Sigal</a>
 * @version $Rev$
 * <p>
 * Copyright Jul 15, 2010
 * </p>
 */
public class CallbackTestCase extends TestCase
{
   private static Logger log = Logger.getLogger(CallbackServerConfigurationTestCase.class);
   private static boolean firstTime = true;
   
   // remoting server connector
   private Connector connector;
   
   public void setUp() throws Exception
   {
      if (firstTime)
      {
         firstTime = false;
         Logger.getLogger("org.jboss.remoting").setLevel(Level.INFO);
         Logger.getLogger("org.jboss.test.remoting").setLevel(Level.INFO);
         String pattern = "[%d{ABSOLUTE}] [%t] %5p (%F:%L) - %m%n";
         PatternLayout layout = new PatternLayout(pattern);
         ConsoleAppender consoleAppender = new ConsoleAppender(layout);
         Logger.getRootLogger().addAppender(consoleAppender);  
      }

   }
   
   
   public void testCallback() throws Throwable
   {      
      log.info("entering " + getName());
      
      // Set up server.
      String host = InetAddress.getLocalHost().getHostAddress();
      int port = PortUtil.findFreePort(host);
      String locatorURI = getTransport() + "://" + host + ":" + port; 
      InvokerLocator serverLocator = new InvokerLocator(locatorURI);
      HashMap serverConfig = new HashMap();
      serverConfig.put(InvokerLocator.FORCE_REMOTE, "true");
      connector = new Connector(serverLocator, serverConfig);
      connector.create();
      ServerInvocationHandler invocationHandler = new SimpleInvocationHandler();
      connector.addInvocationHandler("sample", invocationHandler);
      connector.start();
      System.out.println("Started remoting server with locator uri of: " + locatorURI);
      
      // Set up client.
      HashMap clientConfig = new HashMap();
      clientConfig.put(InvokerLocator.FORCE_REMOTE, "true");
      Client client = new Client(serverLocator, clientConfig);
      client.connect();
      log.info("client is connected");
      
      // Set up push callbacks.
      SimpleCallbackHandler callbackHandler = new SimpleCallbackHandler();
      client.addListener(callbackHandler, new HashMap(), null, true);
      log.info("client added callback handler");

      // Test invocation.
      Integer response = (Integer) client.invoke(new Integer(17));
      log.info("received response: " + response);
      assertEquals(19, response.intValue());
      
      // Test callback.
      assertEquals(18, callbackHandler.value.intValue());
 
      client.removeListener(callbackHandler);
      client.disconnect();
      connector.stop();
      connector.destroy();
   }

   public void tearDown()
   {
   }
   
   protected String getTransport()
   {
      return "socket";
   }
   
   
   static class SimpleInvocationHandler implements ServerInvocationHandler
   {  
      InvokerCallbackHandler callbackHandler;
      
      public void addListener(InvokerCallbackHandler callbackHandler)
      {
         this.callbackHandler = callbackHandler;
      }

      public Object invoke(InvocationRequest invocation) throws Throwable
      {
         // Send callback;
         int i = ((Integer) invocation.getParameter()).intValue();
         callbackHandler.handleCallback(new Callback(new Integer(i + 1)));
         
         // Return response;
         return new Integer(i + 2);
      }
      
      public void removeListener(InvokerCallbackHandler callbackHandler) {}
      public void setMBeanServer(MBeanServer server) {}
      public void setInvoker(ServerInvoker invoker) {}
   }
   
   
   static class SimpleCallbackHandler implements InvokerCallbackHandler
   {  
      Integer value;
      
      public void handleCallback(Callback callback) throws HandleCallbackException
      {
         log.info("received callback: " + callback.getParameter());
         value = (Integer) callback.getParameter();
      }
   }
}
